package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.Validator;
import inf43.spring2009.tpes.converters.ToDayOfWeekConverter;
import inf43.spring2009.tpes.converters.ToTimeOfDayConverter;
import inf43.spring2009.tpes.formatters.MeetingIntervalFormatter;
import inf43.spring2009.tpes.matchers.ClassroomByNameMatcher;
import inf43.spring2009.tpes.matchers.CourseByIdMatcher;
import inf43.spring2009.tpes.model.Classroom;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.CourseMeeting;
import inf43.spring2009.tpes.model.CourseMeetingOverlapsException;
import inf43.spring2009.tpes.model.DayOfWeek;
import inf43.spring2009.tpes.model.MeetingInterval;
import inf43.spring2009.tpes.model.NoSuchClassroomException;
import inf43.spring2009.tpes.model.NoSuchCourseException;
import inf43.spring2009.tpes.model.TimeOfDay;
import inf43.spring2009.tpes.model.TpesData;
import inf43.spring2009.tpes.validators.ClassroomNameExistsValidator;
import inf43.spring2009.tpes.validators.CourseIdExistsValidator;
import inf43.spring2009.tpes.validators.IsNotAfterTimeOfDayValidator;
import inf43.spring2009.tpes.validators.IsNotBeforeTimeOfDayValidator;
import inf43.spring2009.tpes.validators.IsNotWeekendDayOfWeekValidator;
import java.util.ArrayList;
import java.util.List;


public class ScheduleCourseCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String courseId = Prompt.read(
			"Course ID",
			"Enter the ID of the course to be scheduled",
			new TrimStringConverter(),
			new CourseIdExistsValidator(model, "There is no course with that ID"));

		String classroomName = Prompt.read(
			"Classroom Name",
			"Enter the name of the classroom to be scheduled",
			new TrimStringConverter(),
			new ClassroomNameExistsValidator(model, "There is no classroom with that name"));
		
		DayOfWeek meetingDay = Prompt.read(
			"Day of Week",
			"Enter the day of the week on which it should be scheduled (examples: M, MO, MON, TH, THURS)",
			new ToDayOfWeekConverter(),
			new IsNotWeekendDayOfWeekValidator("Classes are not permitted to meet on weekends"));
		
		TimeOfDay startTime = Prompt.read(
			"Start Time",
			"Enter the starting time for the scheduled course meeting, in the form HH:MM (e.g., 17:50)",
			new ToTimeOfDayConverter(),
			createStartTimeValidators());

		TimeOfDay endTime = Prompt.read(
			"End Time",
			"Enter the ending time for the scheduled course meeting, in the form HH:MM (e.g., 17:50)",
			new ToTimeOfDayConverter(),
			createEndTimeValidators(startTime));

		Course course;
		Classroom classroom;
		CourseMeeting courseMeeting;
		
		try
		{
			course = model.getCourses().findCourse(
				new CourseByIdMatcher(courseId));
		}
		catch (NoSuchCourseException e)
		{
			throw new CommandFailedException("There is no course with that ID");
		}

		try
		{
			classroom = model.getClassrooms().findClassroom(
				new ClassroomByNameMatcher(classroomName));
		}
		catch (NoSuchClassroomException e)
		{
			throw new CommandFailedException("There is no classroom with that name");
		}

		courseMeeting = new CourseMeeting(
			course, classroom,
			new MeetingInterval(meetingDay, startTime, endTime));

		try
		{
			course.addCourseMeeting(courseMeeting);
		}
		catch (CourseMeetingOverlapsException e)
		{
			throw new CommandFailedException(
				buildOverlappingMeetingsErrorMessage(e));
		}
		
		try
		{
			classroom.addCourseMeeting(courseMeeting);
		}
		catch (CourseMeetingOverlapsException e)
		{
			course.removeCourseMeeting(courseMeeting);

			throw new CommandFailedException(
				buildOverlappingMeetingsErrorMessage(e));
		}
		
		Console.writeLine(course.getName() + " scheduled into " + classroom.getName());
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
	
	
	private List<? extends Validator<TimeOfDay>> createStartTimeValidators()
	{
		ArrayList<Validator<TimeOfDay>> validators = new ArrayList<Validator<TimeOfDay>>();
		
		validators.add(new IsNotBeforeTimeOfDayValidator(new TimeOfDay(7, 0), "Classes cannot begin before 07:00"));
		validators.add(new IsNotAfterTimeOfDayValidator(new TimeOfDay(22, 0), "Classes cannot begin after 22:00"));

		return validators;
	}
	
	
	private List<? extends Validator<TimeOfDay>> createEndTimeValidators(TimeOfDay startTime)
	{
		ArrayList<Validator<TimeOfDay>> validators = new ArrayList<Validator<TimeOfDay>>();
		
		validators.add(new IsNotBeforeTimeOfDayValidator(new TimeOfDay(7, 0), "Classes cannot begin before 07:00"));
		validators.add(new IsNotAfterTimeOfDayValidator(new TimeOfDay(22, 0), "Classes cannot begin after 22:00"));
		validators.add(new IsNotBeforeTimeOfDayValidator(startTime, "Classes cannot end before they begin"));
		
		return validators;
	}
	
	
	private String buildOverlappingMeetingsErrorMessage(CourseMeetingOverlapsException e)
	{
		return e.getExistingMeeting().getCourse().getName()
			+ " is already meeting in "
			+ e.getExistingMeeting().getClassroom().getName()
			+ " on "
			+ new MeetingIntervalFormatter().format(e.getExistingMeeting().getMeetingInterval());
	}
}
