package inf43.spring2009.tpes.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


public class Course implements Serializable
{
	private String id;
	private String name;
	private int maximumCapacity;
	private int minimumNumberOfMinutes;
	private List<CourseMeeting> courseMeetings;
	private List<Student> enrolledStudents;
	
	
	public Course(String id, String name, int maximumCapacity, int minimumNumberOfMinutes)
	{
		setId(id);
		setName(name);
		setMaximumCapacity(maximumCapacity);
		setMinimumNumberOfMinutes(minimumNumberOfMinutes);
		courseMeetings = new ArrayList<CourseMeeting>();
		enrolledStudents = new ArrayList<Student>();
	}
	
	
	public String getId()
	{
		return id;
	}
	
	
	private void setId(String id)
	{
		if (id == null || id.length() != 4 ||
			!Character.isDigit(id.charAt(0)) ||
			!Character.isDigit(id.charAt(1)) ||
			!Character.isDigit(id.charAt(2)) ||
			!Character.isDigit(id.charAt(3)))
		{
			throw new IllegalArgumentException("id must be a four-digit number");
		}
		
		this.id = id;
	}
	
	
	public String getName()
	{
		return name;
	}
	
	
	private void setName(String name)
	{
		if (name == null || name.trim().length() == 0)
		{
			throw new IllegalArgumentException("name must be non-empty");
		}
		
		this.name = name.trim();
	}
	
	
	public int getMaximumCapacity()
	{
		return maximumCapacity;
	}
	
	
	private void setMaximumCapacity(int maximumCapacity)
	{
		if (maximumCapacity <= 0)
		{
			throw new IllegalArgumentException("maximumCapacity must be positive");
		}
		
		this.maximumCapacity = maximumCapacity;
	}
	
	
	public int getEffectiveCapacity()
	{
		int effectiveCapacity = maximumCapacity;
		
		for (CourseMeeting courseMeeting : courseMeetings)
		{
			if (courseMeeting.getClassroom().getCapacity() < effectiveCapacity)
			{
				effectiveCapacity = courseMeeting.getClassroom().getCapacity();
			}
		}
		
		return effectiveCapacity;
	}
	
	
	public int getMinimumNumberOfMinutes()
	{
		return minimumNumberOfMinutes;
	}
	
	
	private void setMinimumNumberOfMinutes(int minimumNumberOfMinutes)
	{
		if (minimumNumberOfMinutes < 0)
		{
			throw new IllegalArgumentException("minimumNumberOfMinutes must be non-negative");
		}
		
		this.minimumNumberOfMinutes = minimumNumberOfMinutes;
	}
	
	
	public List<CourseMeeting> getCourseMeetings()
	{
		return Collections.unmodifiableList(courseMeetings);
	}
	
	
	public void addCourseMeeting(CourseMeeting courseMeetingToAdd)
	throws CourseMeetingOverlapsException
	{
		if (courseMeetingToAdd == null)
		{
			throw new IllegalArgumentException("courseMeetingToAdd cannot be null");
		}
		
		if (courseMeetingToAdd.getCourse() != this)
		{
			throw new IllegalArgumentException(
				"courseMeetingToAdd refers to wrong classroom");
		}
		
		for (CourseMeeting courseMeeting : courseMeetings)
		{
			if (courseMeeting.getMeetingInterval().overlaps(courseMeetingToAdd.getMeetingInterval()))
			{
				throw new CourseMeetingOverlapsException(courseMeeting, courseMeetingToAdd);
			}
		}
		
		courseMeetings.add(courseMeetingToAdd);
	}
	
	
	public void removeCourseMeeting(CourseMeeting courseMeeting)
	{
		if (courseMeeting == null)
		{
			throw new IllegalArgumentException("courseMeeting cannot be null");
		}
		
		if (courseMeetings.contains(courseMeeting))
		{
			courseMeetings.remove(courseMeeting);
		}
	}
	
	
	public int getScheduledNumberOfMinutes()
	{
		int scheduledMinutes = 0;
		
		for (CourseMeeting courseMeeting : courseMeetings)
		{
			scheduledMinutes += courseMeeting.getMeetingInterval().getLengthInMinutes();
		}

		return scheduledMinutes;
	}
	
	
	public SchedulingStatus getSchedulingStatus()
	{
		int scheduledMinutes = getScheduledNumberOfMinutes();
		
		if (scheduledMinutes == 0)
		{
			return SchedulingStatus.UNSCHEDULED;
		}
		else if (scheduledMinutes < minimumNumberOfMinutes)
		{
			return SchedulingStatus.PARTIALLY_SCHEDULED;
		}
		else
		{
			return SchedulingStatus.FULLY_SCHEDULED;
		}
	}
	
	
	public List<Student> getEnrolledStudents()
	{
		return Collections.unmodifiableList(enrolledStudents);
	}
	
	
	public void enrollStudent(Student studentToEnroll)
	throws CourseFullException
	{
		if (studentToEnroll == null)
		{
			throw new IllegalArgumentException("studentToEnroll must not be null");
		}
		
		if (enrolledStudents.size() < getEffectiveCapacity())
		{
			enrolledStudents.add(studentToEnroll);
		}
		else
		{
			throw new CourseFullException();
		}
	}
	
	
	public void dropStudent(Student studentToDrop)
	throws NotEnrolledException
	{
		if (studentToDrop == null)
		{
			throw new IllegalArgumentException("studentToDrop must not be null");
		}
		
		if (enrolledStudents.contains(studentToDrop))
		{
			enrolledStudents.remove(studentToDrop);
		}
		else
		{
			throw new NotEnrolledException();
		}
	}
}
